package genList;

/**
 * Non-empty list of elements of type E
 */
public class ConsList<E> extends List<E> {
  /** The first element of this List<E>. */
  private E first;
  /** The remaining elements of this List<E>. */
  private List<E> rest;
  
  /**
   * Constructor for the list.  
   * Usage:  List<MyType> aListOfMyType = new List<MyType>(aMyType, anotherListOfMyType);
   * @param first The first element of the list
   * @rest the rest of the list
   */
  public ConsList(E first, List<E> rest) {
    this.first = first;
    this.rest = rest;
  }
  
  /**
   * Accessor for the first element
   */
  public E getFirst() {
    return first;
  }
  
  /**
   * Accessor for the rest of the list
   */
  public List<E> getRest() {
    return rest;
  }
  
  /**
   * Concrete method to accept the given visitor with the given input parameter(s) and return type. Always calls the forCons() method of the visitor.
   * @param <R> the return type of the visitor
   * @param <P> the input parameter type of the visitor
   * @param visitor a visitor to this type of list that takes the given type of input parameters and returns the given type of return value.
   * @param params  Vararg of input parameters to hand to the visitor.
   * @return the result of running the visitor on this list.
   */
  public <R, P> R accept(IListVisitor<E, R, P> visitor, P... params) {
    return visitor.forCons(this, params);
  }

  
}