package genList;

/**
 * Visitor to List&lt;E&gt;
 * @param <E> the type of the data held by the List&lt;E&gt; host.
 * @param <R> the return type of this algorithm.
 * @param <P> the input parameter type used by this algorithm.  Use type Object if not used.
 */
public interface IListVisitor<E, R, P> {
  
  /**
   * Empty list case of the visitor
   * @param host the EmptyList&lt;E&gt; host for this visitor
   * @param params The input parameters, all of type P if any.
   * @return the result of running this visitor on the host list.
   */
  public R forEmpty(EmptyList<E> host, P... params);
  
  /**
   * Non-empty list case of the visitor
   * @param host the ConsList&lt;E&gt; host for this visitor
   * @param params The input parameters, all of type P if any.
   * @return the result of running this visitor on the host list.
   */
  public R forCons(ConsList<E> host, P... params);
  
}
