package genList;

/**
 * Abstract, generic list of elements of type E.
 * @param <E> the type of the data held by this list
 */
public abstract class List<E> { 
 
  /**
   * Concrete method to return a new list with the given element added to the front of this list
   * @param data the data element to use as the new first element.
   * @return a new list of type List&lt;E&gt; with data as its first element and this list as its rest.
   */
  public List<E> cons(E data) {
    return new ConsList<E>(data, this);
  }
  
  /**
   * Abstract method to accept the given visitor with the given input parameter(s) and return type.
   * @param <R> the return type of the visitor
   * @param <P> the input parameter type of the visitor
   * @param visitor a visitor to this type of list that takes the given type of input parameters and returns the given type of return value.
   * @param params  Vararg of input parameters to hand to the visitor.
   * @return the result of running the visitor on this list.
   */
  abstract public <R, P> R accept(IListVisitor<E, R, P> visitor, P... params);
}

