package genList;

/**
 * Visitor to List&lt;Integer&gt; that sums all the elements
 * The input parameter is not used.
 */
public class SumIntListVisitor implements IListVisitor<Integer, Integer, Object> {
  
  public static final SumIntListVisitor ONLY = new SumIntListVisitor();
  
  private SumIntListVisitor() {}
  /**
   * Empty list case of the visitor.   
   * @param host the EmptyList&lt;Integer&gt; host for this visitor
   * @param params not used
   * @return zero, always
   */
  public Integer forEmpty(EmptyList<Integer> host, Object... nu) {
    return 0;
  }
  
  /**
   * Non-empty list case of the visitor.  Add the first of the list to the sum of the rest
   * @param host the ConsList&lt;Integer&gt; host for this visitor
   * @param params not used
   * @return the host's first plus the sum of its rest.
   */
  public Integer forCons(ConsList<Integer> host, Object... nu) {
    return host.getFirst() + host.getRest().accept(this);
  }
  
 
}